#!/bin/bash
#
# Auto generate properties files out of po files
#
# Author: Dominik Brämer <thedsweb@googlemail.com>
#
# Please remove the "Author" lines above and replace them
# with your own name if you copy and modify this script.

CLITEMPLATE=./po/yawls_template.pot
GUITEMPLATE=./po/yawls_gui_template.pot
MANTEMPLATE=./po/yawls_man_template.pot

#Check Translation for empty or fuzzy strings -------------------------------------------
function checkPoFile() {
	COUNT=0
	
	while read LINE; do
  	if [[ "$LINE" == 'msgstr ""' ]]; then
    	read LINE
    	[[ -z "$LINE" ]] && ((COUNT++))
    fi
    
    if [[ "$LINE" == 'fuzzy' ]]; then
    	((COUNT++))
  	fi
  done <$1
  
  PERC=$((100*$COUNT/$MSGCOUNT))
  
  if [[ "$PERC" -lt "20" ]]; then
  	return 0
	else
		return 1
	fi
}

#Concat Last-Translators ----------------------------------------------------------------
AUTHORS=""

function translators() {
	LT=$(grep 'Last-Translator:' "$1" | grep -o ' .* ')
	LT=${LT# }
	LT=${LT% }

	if [[ "$(echo "$AUTHORS" | grep "$LT")" == "" ]] && [[ "$LT" != "FULL NAME" ]]; then
		if [[ "$AUTHORS" == "" ]]; then
			AUTHORS="$LT"
		else
			AUTHORS="$AUTHORS\n$LT"
		fi
	fi
}

#Prepare cli translation ----------------------------------------------------------------

#Delete old translations
rm -f ./src/com/blogspot/thedsweb/local/*.properties

#Upate template
xgettext './src/com/blogspot/thedsweb/main/Yawls.java' -k_ -o $CLITEMPLATE
sed 's/CHARSET/UTF-8/g' -i $CLITEMPLATE

msgcat -p $CLITEMPLATE -o ./src/com/blogspot/thedsweb/local/yawls_template.properties

for i in ./po/yawls_template-*.po; do
	msgmerge -q -U $i $CLITEMPLATE
	
	MSGCOUNT=$(grep -c 'msgid' $i)
	
	if checkPoFile $i; then
		translators $i
		
		FILE=./src/com/blogspot/thedsweb/local/$(basename ${i%.*} | sed 's/-/_/g').properties
		msgcat -p $i -o $FILE
		sed -i 's/\\\\/\\/g' $FILE
	fi
done

#Prepare GUI translation ----------------------------------------------------------------

#Delete old translations
rm -f -r usr

#Upate template
xgettext './yawls_calibrate.glade' './yawls_error.glade' './yawls_gui.glade' -o $GUITEMPLATE
sed 's/CHARSET/UTF-8/g' -i $GUITEMPLATE

for i in ./po/yawls_gui_template-*.po; do
	msgmerge -q -U $i $GUITEMPLATE
	
	MSGCOUNT=$(grep -c 'msgid' $i)
	
	if checkPoFile $i; then
		translators $i
		LOCAL=$(basename ${i%.*})
		LOCAL=${LOCAL#*-}
		
		if [[ $LOCAL == "*" ]]; then
			break
		fi 
		
		FILE=./usr/share/locale/$LOCAL/LC_MESSAGES/yawls.mo
		mkdir -p ./usr/share/locale/$LOCAL/LC_MESSAGES
		msgfmt $i -o $FILE
	fi
done

#Prepare man page translation -----------------------------------------------------------

#Delete old man page lists
rm -f ./debian/yawls-*.manpages
rm -f ./man/yawls.*.1

#Create new man page lists
echo man/yawls.1 >> ./debian/yawls-cli.manpages
echo man/yawls.1 >> ./debian/yawls-gtk.manpages

#Upate template
po4a-gettextize -f man -M UTF-8 -m ./man/yawls.1 -p $MANTEMPLATE

COMMENTS=""

for i in ./po/yawls_man_template-*.po; do
	LOCAL=$(basename ${i%.*})
	LOCAL=${LOCAL#*-}
	
	if [[ $LOCAL == "*" ]]; then
		break
	fi
	
	po4a-updatepo --msgmerge-opt -q -f man -M UTF-8 -m ./man/yawls.1 -p $i
	
	MSGCOUNT=$(grep -c 'msgid' $i)
	
	if checkPoFile $i; then
		translators $i
		
		DIS=$(grep -A 1 'msgid "Yawls automatic display brightness"' "$i" | grep 'msgstr')
		DIS=${DIS#*\"}
		DIS=${DIS%\"}
		
		if [[ "$DIS" != "" ]]; then
			if [[ "$COMMENTS" == "" ]]; then
				COMMENTS="Comment[$LOCAL]=$DIS"
			else
				COMMENTS="$COMMENTS\nComment[$LOCAL]=$DIS"
			fi
		fi
		
		if [[ "$(echo "$LOCAL" | grep "en_*")" == "" ]]; then
			po4a-translate -f man -M UTF-8 -m ./man/yawls.1 -p $i -l ./man/yawls.$LOCAL.1
			
			echo man/yawls.$LOCAL.1 >> ./debian/yawls-cli.manpages
			echo man/yawls.$LOCAL.1 >> ./debian/yawls-gtk.manpages
		fi
	fi
done

#List all translators in about GUI ------------------------------------------------------
sed -e '/<property name="translator_credits">/,/<\/property>/ d' \
-e '/<property name="authors">.*<\/property>/a <property name="translator_credits">'"$AUTHORS"'<\/property>' \
-i './yawls_about.glade'

#Add translated discription to menu file ------------------------------------------------
sed -e '/^Comment\[/ d' \
-e '/Comment=Yawls automatic display brightness/a '"$COMMENTS" \
-i './Yawls.desktop'
