package com.blogspot.thedsweb.gui;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.text.ParseException;
import java.util.Properties;

import org.freedesktop.bindings.Internationalization;
import org.gnome.gdk.Event;
import org.gnome.gdk.Pixbuf;
import org.gnome.gtk.Builder;
import org.gnome.gtk.Button;
import org.gnome.gtk.Button.Clicked;
import org.gnome.gtk.Dialog;
import org.gnome.gtk.Dialog.Response;
import org.gnome.gtk.Entry;
import org.gnome.gtk.Gtk;
import org.gnome.gtk.ImageMenuItem;
import org.gnome.gtk.Label;
import org.gnome.gtk.MenuItem;
import org.gnome.gtk.MenuItem.Activate;
import org.gnome.gtk.ResponseType;
import org.gnome.gtk.Widget;
import org.gnome.gtk.Window;
import org.gnome.gtk.Window.DeleteEvent;

import com.blogspot.thedsweb.engine.Brightness;
import com.blogspot.thedsweb.util.Check;
import com.blogspot.thedsweb.util.Config;
import com.blogspot.thedsweb.util.Database;
import com.blogspot.thedsweb.util.Files;
import com.blogspot.thedsweb.util.Initialization;

public class Show {
	public static void gui(String[] args) {
		Gtk.init(args);
		final Builder mainGUI = new Builder();
		try {
			//Create main window for the GUI and set some general elements
			Gtk.setProgramName("Yawls");
			
			//Use GTK+ translation method
			Internationalization.init("yawls", "/usr/share/locale/");
			
			mainGUI.addFromFile(Database.PATHMAIN);
			final Window mainWindow = (Window)mainGUI.getObject("yawls_gui");
			
			final Pixbuf icon = new Pixbuf(Database.PATHICON);
			mainWindow.setIcon(icon);			
			
			//Get value output
			int[] arr = new int[4];
			Files.readStatus(arr);
			
			//Set maxValue
			Label maxValue = (Label)mainGUI.getObject("maxValue");
			maxValue.setLabel(Integer.toString(arr[1]));
			
			//Set minValue
			Label minValue = (Label)mainGUI.getObject("minValue");
			minValue.setLabel(Integer.toString(arr[0]));
			
			//Set lastValue
			Label lastValue = (Label)mainGUI.getObject("lastValue");
			lastValue.setLabel(Integer.toString(arr[2]));
			
			//Set entry output
			try {
				Config loadConfig = new Config();
				
				//Set the entry for faceDetect
				Entry faceEntry =  (Entry)mainGUI.getObject("face_entry");
				faceEntry.setText(Boolean.toString(loadConfig.faceDetect()));
				
				//Set the entry for lowerLimit
				Entry lowerLimitEntry =  (Entry)mainGUI.getObject("lower_limit_entry");
				lowerLimitEntry.setText(Integer.toString(loadConfig.lowerLimit()));
				
				//Set the entry for upperLimit
				Entry upperLimitEntry =  (Entry)mainGUI.getObject("upper_limit_entry");
				upperLimitEntry.setText(Integer.toString(loadConfig.upperLimit()));
				
				//Set the entry for minIncrease
				Entry minIncreaseEntry =  (Entry)mainGUI.getObject("min_increase_entry");
				minIncreaseEntry.setText(Integer.toString(loadConfig.minIncrease()));
				
				//Set the entry for maxIncrease
				Entry maxIncreaseEntry =  (Entry)mainGUI.getObject("max_increase_entry");
				maxIncreaseEntry.setText(Integer.toString(loadConfig.maxIncrease()));
				
				//Set the entry for darkeningThreshold
				Entry darkThresHoldEntry =  (Entry)mainGUI.getObject("dark_threshold_entry");
				darkThresHoldEntry.setText(Integer.toString(loadConfig.darkeningThreshold()));
				
				//Set the entry for brighteningThreshold
				Entry brightThresHoldEntry =  (Entry)mainGUI.getObject("bright_threshold_entry");
				brightThresHoldEntry.setText(Integer.toString(loadConfig.brighteningThreshold()));
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}
			
			//Action of reload button which reload all displayed values
			Button refresh = (Button)mainGUI.getObject("refresh_button");
			refresh.connect(new Clicked() {
				@Override
				public void onClicked(Button arg0) {
					//Get and set value output
					int[] arr = new int[4];
					Files.readStatus(arr);
					
					Label maxValue = (Label)mainGUI.getObject("maxValue");
					maxValue.setLabel(Integer.toString(arr[1]));
					
					Label minValue = (Label)mainGUI.getObject("minValue");
					minValue.setLabel(Integer.toString(arr[0]));
					
					Label lastValue = (Label)mainGUI.getObject("lastValue");
					lastValue.setLabel(Integer.toString(arr[2]));
				}
			});
			
			//Function to close the program
			mainWindow.connect(new DeleteEvent() {
				@Override
				public boolean onDeleteEvent(Widget arg0, Event arg1) {
					Gtk.mainQuit();
					return false;
				}
			});
			
			//Action of restore button
			Button restore = (Button)mainGUI.getObject("restore_button");
			restore.connect(new Clicked() {
				@Override
				public void onClicked(Button arg0) {
					if ( Check.root() ) {
						Initialization.forceFiles();
					} else {
						Builder errorGUI = new Builder();
						try {
							errorGUI.addFromFile(Database.PATHERROR);
							final Window errorWindow = (Window)errorGUI.getObject("yawls_error");
							errorWindow.setIcon(icon);
							
							//Action of cancel button
							Dialog closeAbout = (Dialog)errorGUI.getObject("yawls_error");
							closeAbout.connect(new Response() {
								@Override
								public void onResponse(Dialog arg0, ResponseType arg1) {
									errorWindow.hide();
								}
							});
							
							//Show error GUI
							errorWindow.showAll();
						} catch (FileNotFoundException e) {
							e.printStackTrace();
						} catch (ParseException e) {
							e.printStackTrace();
						}
					}	
				}
			});
			
			//Action of apply button
			Button apply = (Button)mainGUI.getObject("apply_button");
			apply.connect(new Clicked() {
				@Override
				public void onClicked(Button arg0) {
					if ( Check.root() ) {
						//Get the entry for faceDetect
						Entry faceEntry =  (Entry)mainGUI.getObject("face_entry");
						String faceDetect = faceEntry.getText();
						
						//Get the entry for lowerLimit
						Entry lowerLimitEntry =  (Entry)mainGUI.getObject("lower_limit_entry");
						String lowerLimit = lowerLimitEntry.getText();
						
						//Get the entry for upperLimit
						Entry upperLimitEntry =  (Entry)mainGUI.getObject("upper_limit_entry");
						String upperLimit = upperLimitEntry.getText();
						
						//Get the entry for minIncrease
						Entry minIncreaseEntry =  (Entry)mainGUI.getObject("min_increase_entry");
						String minIncrease = minIncreaseEntry.getText();
						
						//Get the entry for maxIncrease
						Entry maxIncreaseEntry =  (Entry)mainGUI.getObject("max_increase_entry");
						String maxIncrease = maxIncreaseEntry.getText();
						
						//Get the entry for darkThresHold
						Entry darkThresHoldEntry =  (Entry)mainGUI.getObject("dark_threshold_entry");
						String darkThresHold = darkThresHoldEntry.getText();
						
						//Get the entry for brightThresHold
						Entry brightThresHoldEntry =  (Entry)mainGUI.getObject("bright_threshold_entry");
						String brightThresHold = brightThresHoldEntry.getText();
						
						//Create new properties object to store values
						Properties config = new Properties();
						
						try {
							Config loadConfig = new Config();
							
							//Check consistency of faceDetect value
							if ( !faceDetect.equals("") && faceDetect.equals("true") || faceDetect.equals("false") ) {
								config.setProperty("faceDetect", faceDetect);
							} else {
								config.setProperty("faceDetect", Boolean.toString(loadConfig.faceDetect()));
								faceEntry.setText("");
							}
							
							//Check consistency of lowerLimit value
							if ( !lowerLimit.equals("") && Integer.parseInt(lowerLimit) >= 0 && Integer.parseInt(lowerLimit) <= 600000 ) {
								config.setProperty("lowerLimit", lowerLimit);
							} else {
								config.setProperty("lowerLimit", Integer.toString(loadConfig.lowerLimit()));
								lowerLimitEntry.setText("");
							}
							
							//Check consistency of upperLimit value
							if ( !upperLimit.equals("") && Integer.parseInt(upperLimit) >= 0 && Integer.parseInt(upperLimit) <= 600000 ) {
								config.setProperty("upperLimit", upperLimit);
							} else {
								config.setProperty("upperLimit", Integer.toString(loadConfig.upperLimit()));
								upperLimitEntry.setText("");
							}
							
							//Check consistency of minIncrease value
							if ( !minIncrease.equals("") && Integer.parseInt(minIncrease) >= 0 && Integer.parseInt(minIncrease) <= 600000 ) {
								config.setProperty("minIncrease", minIncrease);
							} else {
								config.setProperty("minIncrease", Integer.toString(loadConfig.minIncrease()));
								minIncreaseEntry.setText("");
							}
							
							//Check consistency of maxIncrease value
							if ( !maxIncrease.equals("") && Integer.parseInt(maxIncrease) >= 0 && Integer.parseInt(maxIncrease) <= 600000 ) {
								config.setProperty("maxIncrease", maxIncrease);
							} else {
								config.setProperty("maxIncrease", Integer.toString(loadConfig.maxIncrease()));
								maxIncreaseEntry.setText("");
							}
							
							//Check consistency of darkThresHold value
							if ( !darkThresHold.equals("") && Integer.parseInt(darkThresHold) >= 0 && Integer.parseInt(darkThresHold) <= 100 ) {
								config.setProperty("darkeningThreshold", darkThresHold);
							} else {
								config.setProperty("darkeningThreshold", Integer.toString(loadConfig.darkeningThreshold()));
								darkThresHoldEntry.setText("");
							}
							
							//Check consistency of maxIncrease value
							if ( !brightThresHold.equals("") && Integer.parseInt(brightThresHold) >= 0 && Integer.parseInt(brightThresHold) <= 600000 ) {
								config.setProperty("brighteningThreshold", brightThresHold);
							} else {
								config.setProperty("brighteningThreshold", Integer.toString(loadConfig.brighteningThreshold()));
								brightThresHoldEntry.setText("");
							}
							
							 File f = new File(Database.PATHTOCONFIGFILE);
					        try {
								OutputStream configStream = new FileOutputStream( f );
								config.store(configStream, "Yawls auto generated config");
							} catch (IOException e) {
								e.printStackTrace();
							}
					        
					        //Create a lock file to trigger the daemon to reload the configuration on the fly
							final File file = new File(Database.PATHLOCKCONFIG);
							try {
								file.createNewFile();
							} catch (IOException e) {
								e.printStackTrace();
							}
						} catch (MalformedURLException e) {
							e.printStackTrace();
						} 
					} else {
						Builder errorGUI = new Builder();
						try {
							errorGUI.addFromFile(Database.PATHERROR);
							final Window errorWindow = (Window)errorGUI.getObject("yawls_error");
							errorWindow.setIcon(icon);
							
							//Action of cancel button
							Dialog closeAbout = (Dialog)errorGUI.getObject("yawls_error");
							closeAbout.connect(new Response() {
								@Override
								public void onResponse(Dialog arg0, ResponseType arg1) {
									errorWindow.hide();
								}
							});
							
							//Show error GUI
							errorWindow.showAll();
						} catch (FileNotFoundException e) {
							e.printStackTrace();
						} catch (ParseException e) {
							e.printStackTrace();
						}
					}
				}
			});
			
			//Function to close the program
			mainWindow.connect(new DeleteEvent() {
				@Override
				public boolean onDeleteEvent(Widget arg0, Event arg1) {
					Gtk.mainQuit();
					return false;
				}
			});
			
			//Action of help menu item
			ImageMenuItem aboutMenuItem = (ImageMenuItem)mainGUI.getObject("about_item");
			aboutMenuItem.connect(new Activate() {
				@Override
				public void onActivate(MenuItem arg0) {
					//Create about GUI
					Builder aboutGUI = new Builder();
					try {
						aboutGUI.addFromFile(Database.PATHABOUT);
						final Window aboutWindow = (Window)aboutGUI.getObject("yawls_about");
						aboutWindow.setIcon(icon);
						//Action of close button
						Dialog closeAbout = (Dialog)aboutGUI.getObject("yawls_about");
						closeAbout.connect(new Response() {
							@Override
							public void onResponse(Dialog arg0, ResponseType arg1) {
								aboutWindow.hide();
							}
						});
						
						//Show about GUI
						aboutWindow.showAll();
					} catch (FileNotFoundException e) {
						e.printStackTrace();
					} catch (ParseException e) {
						e.printStackTrace();
					}
				}
			});
			
			//Action of question menu item
			ImageMenuItem questionMenuItem = (ImageMenuItem)mainGUI.getObject("question_item");
			questionMenuItem.connect(new Activate() {
				@Override
				public void onActivate(MenuItem arg0) {
					//Open launchpad.net project questions web page
					if(java.awt.Desktop.isDesktopSupported() ) {
				        java.awt.Desktop desktop = java.awt.Desktop.getDesktop();

				        if(desktop.isSupported(java.awt.Desktop.Action.BROWSE) ) {
				        	java.net.URI uri;
							try {
								//Set URL for launchpad.net FAQ
								uri = new java.net.URI(com.blogspot.thedsweb.util.Database.QUESTIONS);
								desktop.browse(uri);
							} catch (URISyntaxException e) {
								e.printStackTrace();
							} catch (IOException e) {
								e.printStackTrace();
							}
				        }
					}
				}
			});
			
			//Action of preference menu item
			ImageMenuItem calibMenuItem = (ImageMenuItem)mainGUI.getObject("calib_item");
			calibMenuItem.connect(new Activate() {
				@Override
				public void onActivate(MenuItem arg0) {
					//Create calibrate GUI
					if ( Check.root() ) {
						//Create a lock file to halt the daemon
						final File file = new File(Database.PATHLOCKDAEMON);
						try {
							file.createNewFile();
							Runtime.getRuntime().addShutdownHook(new Thread() {
								public void run() {
									try {
										file.delete();
									} catch ( Exception e ) {
										throw new RuntimeException();
									}
								}
							});
						} catch ( IOException e ) {
							e.printStackTrace();
						}
						
						final Builder calibrateGUI = new Builder();
						try {
							calibrateGUI.addFromFile(Database.PATHCALIB);
							final Window calibrateWindow = (Window)calibrateGUI.getObject("yawls_calibrate");
							calibrateWindow.setIcon(icon);
							
							//Cancel button action
							Button cancleCali = (Button)calibrateGUI.getObject("close_button");
							cancleCali.connect(new Clicked() {
								@Override
								public void onClicked(Button arg0) {
									//Delete the lock file and trigger the daemon to reload the calibration
									file.delete();
									calibrateWindow.hide();
								}
							});
							
							//Action of close window
							Dialog closeCali = (Dialog)calibrateGUI.getObject("yawls_calibrate");
							closeCali.connect(new Response() {
								@Override
								public void onResponse(Dialog arg0, ResponseType arg1) {
									file.delete();
									calibrateWindow.hide();
								}
							});
							
							//Action of calibration button
							Button getBrightness = (Button)calibrateGUI.getObject("get_brightness_button");
							getBrightness.connect(new Clicked() {
								@Override
								public void onClicked(Button arg0) {
									//Read configuration File
									Config config;
									
									try {
										config = new Config();
										
										//Get current brightness and save it
										Brightness brightness = new Brightness(config);
										
										//Set new brightness level
										int current = brightness.calibrateBrightness();
										
										//Show latest value in GUI
										Label lastValueCalib = (Label)calibrateGUI.getObject("last_value_text");
										lastValueCalib.setLabel(Integer.toString(current));
									} catch (MalformedURLException e) {
										e.printStackTrace();
									}
								}
							});
							
							//Show about GUI
							calibrateWindow.showAll();
						} catch (FileNotFoundException e) {
							e.printStackTrace();
						} catch (ParseException e) {
							e.printStackTrace();
						}
					} else {
						Builder errorGUI = new Builder();
						try {
							errorGUI.addFromFile(Database.PATHERROR);
							final Window errorWindow = (Window)errorGUI.getObject("yawls_error");
							errorWindow.setIcon(icon);
							
							//Action of cancel button
							Dialog closeAbout = (Dialog)errorGUI.getObject("yawls_error");
							closeAbout.connect(new Response() {
								@Override
								public void onResponse(Dialog arg0, ResponseType arg1) {
									errorWindow.hide();
								}
							});
							
							//Show error GUI
							errorWindow.showAll();
						} catch (FileNotFoundException e) {
							e.printStackTrace();
						} catch (ParseException e) {
							e.printStackTrace();
						}
					}
				}
			});
			
			mainWindow.showAll();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (ParseException e) {
			e.printStackTrace();
		}
		Gtk.main();
	}
}
