package com.blogspot.thedsweb.main;

import java.io.File;
import java.net.MalformedURLException;

import com.blogspot.thedsweb.engine.Brightness;
import com.blogspot.thedsweb.engine.Stable;
import com.blogspot.thedsweb.gui.Show;
import com.blogspot.thedsweb.util.Battery;
import com.blogspot.thedsweb.util.Check;
import com.blogspot.thedsweb.util.Config;
import com.blogspot.thedsweb.util.Database;
import com.blogspot.thedsweb.util.Files;
import com.blogspot.thedsweb.util.Initialization;
import com.blogspot.thedsweb.util.SingleInstance;
import com.blogspot.thedsweb.util.Translate;

public class Yawls {
	public static void main(String[] args) {
		//Simple command line argument parsing
		for( String options : args ){
			switch(options) {
			case "-h":
			case "--help":
				help();
				break;
				//
			case "-r":
			case "--restore":
				if ( Check.root() ) {
					restoreConfig();
				} else {
					rootOutput();
				}
				break;
				//
			case "-c":
			case "--cron":
				if ( Check.root() ) {
					try {
						if ( SingleInstance.lockInstance() ) {
							automatic();
						} else {
							singleInstanceOutput();
						}
					} catch (MalformedURLException e) {
						e.printStackTrace();
					}
				} else {
					rootOutput();
				}
				break;
				//
			case "-d":
			case "--daemon":
				if ( Check.root() ) {
					try {
						if ( SingleInstance.lockInstance() ) {
							daemon();
						} else {
							singleInstanceOutput();
						}
					} catch (MalformedURLException e) {
						e.printStackTrace();
					}
				} else {
					rootOutput();
				}
				break;
				//
			case "-g":
			case "--gui":
				if ( Check.gtk() ) {
					Show.gui(args);
				} else {
					System.out.println(Translate._("You should install the GTK version of yawls to use this feature."));
				}
				break;
				//
			default:
				System.out.println(Translate._("Usage: yawls [OPTIONS]"));
				//
			}
		}
		if ( args == null || args.length == 0 ){
			System.out.println(Translate._("Usage: yawls [OPTIONS]"));
		}
	}
	
	public static void singleInstanceOutput() {
		System.out.println(Translate._("Another instance of yawls is already running!"));
	}
	
	public static void rootOutput() {
		System.out.println(Translate._("This command must be run as root"));
	}
	
	public static void help(){
		System.out.println(
				  Translate._("Usage: yawls") + "\n"
				+ Translate._("yawls adjust the brightness level of your display by using\nthe internal/externel webcam of your notebook as an ambient\nlight sensor.")
				+ "\n\n"
				+ Translate._("If you wish to change the interval of ambient light checks\nyou need to modify the file:") + "\n"
				+ "   /etc/cron.d/yawls"
				+ "\n\n"
				+ Translate._("Options:") + "\n"
				+ "  " + Translate._("--cron\t or -c use webcam as brightness reference") + "\n"
				+ "  " + Translate._("--help\t or -h show this help") + "\n"
				+ "  " + Translate._("--restore\t or -r restore config files"));
	}
	
	public static void automatic() throws MalformedURLException {
		//Initialize Files if necessary 
		Initialization.Files();
		
		//Read configuration File
		Config config = new Config();
		
		//Get current brightness and save it
		Brightness brightness = new Brightness(config);
		
		// If the current value change in a extreme way
		// try to set it again
		setNewIfChangeTooExtreme(brightness);
		
		//Set new brightness level
		brightness.setBrightness();
	}
	
	public static void daemon() throws MalformedURLException {
		//Set daemon mode
		System.out.println(Translate._("Yawls service started press {ctrl+c} to abort..."));
		Runtime.getRuntime().addShutdownHook( new Thread() {
			@Override 
			public void run() {
				Files.deletePid();
			}
		} );
		System.out.close();
		System.err.close();
		
		//Initialize Files if necessary 
		Initialization.Files();
		
		// Read configuration File
		// Upper and lower limit between brightness capturing
		Config config = new Config();
		boolean faceDetectConfig = config.faceDetect();
		int lowerLimit = config.lowerLimit();
		int upperLimit = config.upperLimit();
		int[] limit = {lowerLimit,upperLimit,lowerLimit,upperLimit};
		checkLimit(limit);
		int i = limit[2];
		
		//Values to increase loop sleep if nothing to do
		int before = 0; 
		int after = 0;
		int minIncrease = config.minIncrease();
		int maxIncrease = config.maxIncrease();
		int increase = maxIncrease;
		Stable stable = new Stable(after, lowerLimit);
		
		
		//Get current brightness in brightness object
		Brightness brightness = new Brightness(config);
		
		//Initialize variable for face detection
		boolean faceDetect;
		
		//Calibration has been running
		boolean calibration = false;
		
		//Loop until interrupt
		File file = new File(Database.PATHPID);
		while ( file.exists() ) {
			//Reload configuration on change
			if ( Check.configuration() ) {
				config = new Config();
				faceDetectConfig = config.faceDetect();
				lowerLimit = config.lowerLimit();
				upperLimit = config.upperLimit();
				limit[0] = lowerLimit;
				limit[1] = upperLimit;
				limit[2] = lowerLimit;
				limit[3] = upperLimit;
				checkLimit(limit);
				i = limit[2];
				stable = new Stable(after, lowerLimit);
			}
			
			//Check if calibration is running
			if ( Check.calibration() ) {
				if ( !calibration ) {
					//Get current brightness
					brightness.setCurrent();
				} else {
					//Reinitialize brightness after calibration
					calibration = false;
					brightness = new Brightness(config);
				}
			} else {
				calibration = true;
			}
			
			// If the current value change in a extreme way
			// try to set it again
			setNewIfChangeTooExtreme(brightness);
			
			//Get last brightness value BEFORE processing
			before = brightness.getLast();
			
			/* 
			 * If face detection is deactivated then set
			 * faceDetect to true only else set
			 * faceDetect to the value of the getFace function
			 * of Brightness object
			*/ 
			if ( faceDetectConfig ) {
				faceDetect = brightness.getFace();
			} else {
				faceDetect = true;
			}
			
			// Check upper and lower limit and set if
			// necessary a more battery friendly value
			checkLimit(limit);
			lowerLimit = limit[2];
			upperLimit = limit[3];
			
			if ( faceDetect ) {
				//Set brightness level
				brightness.setBrightness();
				upperLimit = limit[1];
				increase = maxIncrease;
			} else {
				//Dim display
				brightness.darkenBrightness();
				increase = minIncrease;
			}
			
			//Get last brightness value AFTER processing
			after = brightness.getLast();
			
			//Increase loop sleep
			if ( before == after ) {
				if ( i < upperLimit ) {
					i = i + increase;
				}
			} else {
				i = lowerLimit;
			}
			
			//Increase correction
			stable.setStatus(i, after);
			i = stable.getLimit();
			
			//Sleep for i seconds
			try {
				Thread.sleep(i);
			} catch ( InterruptedException e ) {}
		}
	}
	
	public static void checkLimit(int[] limit) {
		int lowerLimitConst = limit[0];
		int upperLimitConst = limit[1];
		int lowerLimit = limit[2];
		int upperLimit = limit[3];
		
		//Upper limit correction
		if ( lowerLimitConst > upperLimitConst ) {
			upperLimitConst = lowerLimit;
			upperLimit = lowerLimit;
		}
		
		//Battery friendliness 
		if ( Battery.status() > 30 ) {
			lowerLimit = lowerLimitConst;
			upperLimit = upperLimitConst;
		} else {
			lowerLimit = ((upperLimitConst-lowerLimitConst)/2)+lowerLimitConst;
		}
		
		//Return new upper and lower limit
		limit[1] = upperLimitConst;
		limit[2] = lowerLimit;
		limit[3] = upperLimit;
	}
	
	public static void setNewIfChangeTooExtreme(Brightness brightness) {
		// Set new current if the change between the last and
		// current values is very high 
		if ( brightness.control() ) {
			brightness.setCurrent();
		}
	}
	
	public static void restoreConfig(){
		//Forces initialization of saved files
		Initialization.forceFiles();
	}
}
