package com.blogspot.thedsweb.util;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Locale;
import java.util.ResourceBundle;

import com.blogspot.thedsweb.error.IllegalArgumentInConfig;

public class Config {
	private ResourceBundle resourceBundle;
	
	public Config() throws MalformedURLException {
		//Load resource Bundle from configuration file path
		File file = new File(Database.PATHTOCONFIG);
		URL[] urls = {file.toURI().toURL()};
		ClassLoader loader = new URLClassLoader(urls);
		resourceBundle = ResourceBundle.getBundle("yawls", Locale.getDefault(), loader);
	}

	public boolean faceDetect() {
		//Load faceDetect configuration
		return resourceBundle.getString("faceDetect").equalsIgnoreCase("true");
	}
	
	public int lowerLimit() {
		//Load lowerLimit configuration
		int lowerLimit = Integer.parseInt(resourceBundle.getString("lowerLimit"));
		
		//Check if value greater zero
		if ( lowerLimit <= 0 ) {
			try {
				throw new IllegalArgumentInConfig();
			} catch (IllegalArgumentInConfig e) {
				e.printStackTrace();
			}
		}
		
		return lowerLimit;
	}
	
	public int upperLimit() {
		//Load upperLimit configuration
		int upperLimit = Integer.parseInt(resourceBundle.getString("upperLimit"));
		
		//Check if value greater zero
		if ( upperLimit <= 0 ) {
			try {
				throw new IllegalArgumentInConfig();
			} catch (IllegalArgumentInConfig e) {
				e.printStackTrace();
			}
		}
		
		return upperLimit;
	}
	
	public int minIncrease() {
		//Load minIncrease configuration
		int minIncrease = Integer.parseInt(resourceBundle.getString("minIncrease"));
		
		//Check if value greater zero
		if ( minIncrease <= 0 ) {
			try {
				throw new IllegalArgumentInConfig();
			} catch (IllegalArgumentInConfig e) {
				e.printStackTrace();
			}
		}
		
		return minIncrease;
	}
	
	public int maxIncrease() {
		//Load maxIncrease configuration
		int maxIncrease = Integer.parseInt(resourceBundle.getString("maxIncrease"));
		
		//Check if value greater zero
		if ( maxIncrease <= 0 ) {
			try {
				throw new IllegalArgumentInConfig();
			} catch (IllegalArgumentInConfig e) {
				e.printStackTrace();
			}
		}
		
		return maxIncrease;
	}
	
	public int darkeningThreshold() {
		//Load darkeningThreshold configuration
		int darkeningThreshold = Integer.parseInt(resourceBundle.getString("darkeningThreshold"));
		
		//Check if value is between 0 and 100
		if ( darkeningThreshold < 0  || darkeningThreshold > 100 ) {
			try {
				throw new IllegalArgumentInConfig();
			} catch (IllegalArgumentInConfig e) {
				e.printStackTrace();
			}
		}
		
		return darkeningThreshold;
	}
	
	public int brighteningThreshold() {
		//Load brighteningThreshold configuration
		int brighteningThreshold = Integer.parseInt(resourceBundle.getString("brighteningThreshold"));
		
		//Check if value is between 0 and 100
		if ( brighteningThreshold < 0 || brighteningThreshold > 100 ) {
			try {
				throw new IllegalArgumentInConfig();
			} catch (IllegalArgumentInConfig e) {
				e.printStackTrace();
			}
		}
		
		return brighteningThreshold;
	}
}
